@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2023 Jakub Kruszona-Zawadzki, Saglabs SA
+ * Copyright (C) 2024 Jakub Kruszona-Zawadzki, Saglabs SA
  * 
  * This file is part of MooseFS.
  * 
@@ -22,7 +22,7 @@
 #define _CRC_H_
 #include <inttypes.h>
 
-uint32_t mycrc32(uint32_t crc,const uint8_t *block,uint32_t leng);
+uint32_t mycrc32(uint32_t crc,const void *block,uint32_t leng);
 uint32_t mycrc32_combine(uint32_t crc1, uint32_t crc2, uint32_t leng2);
 #define mycrc32_zeroblock(crc,zeros) mycrc32_combine((crc)^0xFFFFFFFF,0xFFFFFFFF,(zeros))
 #define mycrc32_zeroexpanded(crc,block,leng,zeros) mycrc32_zeroblock(mycrc32((crc),(block),(leng)),(zeros))
