@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2023 Jakub Kruszona-Zawadzki, Saglabs SA
+ * Copyright (C) 2024 Jakub Kruszona-Zawadzki, Saglabs SA
  * 
  * This file is part of MooseFS.
  * 
@@ -25,9 +25,9 @@
 
 int rnd_init(void);
 
-uint8_t rndu8();
-uint32_t rndu32();
-uint64_t rndu64();
+uint8_t rndu8(void);
+uint32_t rndu32(void);
+uint64_t rndu64(void);
 
 void rndbuff(uint8_t *buff,uint32_t size);
 
