@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2023 Jakub Kruszona-Zawadzki, Saglabs SA
+ * Copyright (C) 2024 Jakub Kruszona-Zawadzki, Saglabs SA
  * 
  * This file is part of MooseFS.
  * 
@@ -29,21 +29,24 @@ uint64_t meta_version(void);
 void meta_cleanup(void);
 void meta_setignoreflag(void);
 void meta_allowautorestore(void);
+void meta_emptystart(void);
 void meta_incverboselevel(void);
 void meta_sendall(int socket);
 int meta_downloadall(int socket);
-
-void meta_text_dump(FILE *fd);
+void meta_do_store_metadata(void);
 
 uint64_t meta_get_id(void);
 void meta_set_id(uint64_t newmetaid);
 
 uint8_t meta_mr_setmetaid(uint64_t newmetaid);
 
-void meta_info(uint32_t *lsstore,uint32_t *lstime,uint8_t *lsstat);
+void meta_info(uint32_t *lsstore,uint32_t *lstime,uint8_t *lsstat,uint64_t *lsmetavers,uint32_t *lschecksum);
 int meta_init(void);
 
 int meta_restore(void);
 
+void meta_download_status(uint8_t status);
+
+uint64_t meta_chlog_keep_version(void);
 
 #endif
